#!/usr/bin/env python
# -*- coding: utf-8 -*-

from PyQt4.QtCore import Qt, QString, QSizeF, SIGNAL
from PyKDE4.kdecore import KToolInvocation
from PyQt4.QtGui import QGraphicsLinearLayout
from PyKDE4.plasma import Plasma
from PyKDE4 import plasmascript
import sys, subprocess, os, pwd, grp

class Edubar(plasmascript.Applet):
    def __init__(self,parent,args=None):
        plasmascript.Applet.__init__(self,parent)

        # FIXME: These variables below was created to change the behavior
        # of hoverEnter and hoverLeave events. There would be a better way
        # to do that, and this code should be reviewed in future.
        self.__installIcon = None
        self.__defaultHoverEnterEvent = None
        self.__defaultHoverLeaveEvent = None

    def createIcon(self, imagefile, size, widget=None):

        obj = Plasma.IconWidget()
        obj.setIcon(QString(imagefile))

        obj.setMinimumSize(QSizeF(size,size))

        return obj

    def launchBrowser(self, link):
        """ Open browser using 'link' as parameter. """

        KToolInvocation.invokeBrowser(link)

    def hDomain(self):
        self.launchBrowser("http://www.dominiopublico.gov.br/")

    def hSearch(self):
        cmd = "/usr/bin/le-search"
        subprocess.Popen(cmd,shell=True)

    def hSchool(self):
        self.launchBrowser("http://tvescola.mec.gov.br/")

    def hObjects(self):
        self.launchBrowser("http://objetoseducacionais2.mec.gov.br/")

    def hTeacher(self):
        self.launchBrowser("http://portaldoprofessor.mec.gov.br/")

    def hDownload(self):
        cmd = "kdesudo -d -c \"/usr/local/bin/le-installpkgs\""
        subprocess.Popen(cmd,shell=True)

    def imgPathFromFile(self,imgFile):
        imagespath = self.applet.package().path() + "contents/imgs/"
        return imagespath + imgFile

    def _runCmd(self, cmd):
        """ Run cmd and return a tuple (retcode, output) """

        s = subprocess.Popen(cmd, stdout=subprocess.PIPE,
                             stderr=subprocess.PIPE, shell=True)

        # FIXME: Find out why s.wait() freezes sometimes
        #retcode = s.wait()
        output = s.communicate()
        #if retcode == 0:
        #    output = output[0].split("\n")
        #else:
        #    output = output[1]
        try:
            output = output[0].split("\n")
        except:
            output = []
        retcode = 0

        return (retcode,output)

    def getMECPackagesList(self):
        """ Return all mec- packages installed on the system, or False if
            some error occurred """

        mecpkgsfile = "/etc/le-edubar/mecpackages.list"
        if os.path.isfile(mecpkgsfile):
            pkgs = open(mecpkgsfile,"r").read().split()
        else:
            cmd = "apt-cache search '^mec-'"
            retcode, pkgs = self._runCmd(cmd)
            if (retcode==0):
                # Remove package description, getting only package name
                pkgs = [ i.split()[0] for i in pkgs if i.startswith("mec-") ]
            else:
                self.logMsg("Error getting packages list using: %s" % cmd)
                pkgs = False

        return pkgs

    def getMECPackagesInstalled(self):
        """ Return the list of installed mec- packages, or False if some
            error occurred """

        cmd = "dpkg --get-selections"
        retcode, pkgs = self._runCmd(cmd)

        if (retcode==0):
            # Get only installed packages
            pkgs = [ i.split()[0] for i in pkgs if i.startswith("mec-") ]
        else:
            self.logMsg("Error while getting list of installed packages")
            pkgs = False

        return pkgs

    def getMECPackagesDiff(self):
        """ Returns the difference between existing mec- packages and those
            that are installed on the system """

        existing = set(self.getMECPackagesList())
        installed = set(self.getMECPackagesInstalled())

        return list(existing - installed)

    def logMsg(self, msg):
        sys.stderr.write("LOG: %s\n" % msg)

    def installHoverEnterEvent(self, widget):
        """ Reimplementation of icon.hoverEnterEvent """
        self.__installIcon.setIcon(self.imgPathFromFile("5b.png"))
        self.__defaultHoverEnterEvent(None)

    def installHoverLeaveEvent(self, widget):
        """ Reimplementation of icon.hoverLeaveEvent """
        self.__defaultHoverLeaveEvent(None)
        self.__installIcon.setIcon(self.imgPathFromFile("5.png"))

    def getScreenGeometry(self):
        """ Return screen geometry in QSize format"""

        return self.applet.scene().screenGeometry(0)

    def imAdmin(self):
        """ Check if the current user is member of admin group.
            return True is he is, and False otherwise """

        login = pwd.getpwuid(os.geteuid()).pw_name
        if login in grp.getgrnam("admin").gr_mem:
            return True
        else:
            return False

    def init(self):

        self.setHasConfigurationInterface(False)
        self.setAspectRatioMode(Plasma.IgnoreAspectRatio)
        self.setBackgroundHints(Plasma.Applet.NoBackground)

        mlayout = QGraphicsLinearLayout(Qt.Horizontal, self.applet)
        mlayout.setSpacing(0)

        iDict = {"domain": {"handler":self.hDomain, "img":"1.png"},
                 "search": {"handler":self.hSearch, "img":"0.png"},
                 "school": {"handler":self.hSchool, "img":"2.png"},
                 "teacher": {"handler":self.hTeacher, "img":"3.png"},
                 "objects": {"handler":self.hObjects, "img":"4.png"},
                 "download":{"handler":self.hDownload, "img":"5.png"},
                 "searchC": {"handler":self.hSearch, "img":"0c.png"}
                }

        pkgsInstalled = self.getMECPackagesInstalled()
        pkgsAvailable = self.getMECPackagesList()

        if self.imAdmin():
            if False in [pkgsInstalled,pkgsAvailable]:
                # Some error occurred or no package is installed
                # TODO: use exception to catch this kind of error
                icons = [iDict["download"]]
            else:
                pkgsDiff = set(pkgsAvailable) - set(pkgsInstalled)
                if len(pkgsDiff) == 0:
                    # If all packages are already installed
                    icons = [iDict["search"]]
                elif len(pkgsDiff) > 0:
                    # There are some packages not installed yet
                    icons = [iDict["download"],iDict["searchC"]]
        else:
            # TODO: if there aren't installed packages, this button shouldn't
            # be shown. To do this a new version of "domain" icon, with soft
            # corners, is required.
            icons = [iDict["search"]]

        icons += [iDict["domain"], iDict["school"], iDict["teacher"],
                  iDict["objects"]]

        # Get the screen size and calculate the icon size
        width = self.getScreenGeometry().width()
        if width < 1024:
            wsize = 96
        else:
            wsize = 128

        for iDesc in icons:
            icon = self.createIcon(self.imgPathFromFile(iDesc["img"]),wsize)
            icon.connect(icon,SIGNAL("clicked()"),iDesc["handler"])
            if iDesc["img"] == "5.png":
                # FIXME: There should be a better way to do this...
                self.__installIcon = icon
                self.__defaultHoverEnterEvent = icon.hoverEnterEvent
                self.__defaultHoverLeaveEvent = icon.hoverLeaveEvent
                icon.hoverEnterEvent = self.installHoverEnterEvent
                icon.hoverLeaveEvent = self.installHoverLeaveEvent

            mlayout.addItem(icon)

        self.applet.setLayout(mlayout)
        self.resize(len(icons)*wsize,wsize)

def CreateApplet(parent):
    return Edubar(parent)
